//(function(){

var imagesDir = "images/";
var soundsDir = "audio/";
var preload;
var manifest = [];
var muteSounds = false;

var data;
var questionNum;
var questions;
var correctAnswer;
var correctAnswers;

$(document).ready(function(){
	// hide everything but preloader
	$('#quiz_q_and_a').hide();
	$('#quiz_question_number').hide();
	$('#quiz_image').hide();
	$('#quiz_tracker').hide();
	$('#quiz_popup').hide();
	$('#quiz_audio_toggle').hide();
	$('#quiz_next').hide();

	getJSON();
});

function getJSON(){
	//console.log("loading JSON");
	$.getJSON('quiz.json', function(result){
		data = result;
		//console.log("loaded JSON");
		preloadAssets();
	}).fail(function(jqXHR, textStatus, errorThrown) {
		//console.log("JSON " + errorThrown);
	});
}

// find images in JSON and add to preload manifest
function recurse(key, val) {
	if (val instanceof Object) {
		$.each(val, recurse);
	} else {
		if (val.match(/\.(gif|jpg|jpeg|png)/gi)){
			var duplicate = false;
			for (var i = 0; i < manifest.length; i++){
				//console.log(val + " vs" + String(manifest[i]).split("/").pop());
				if (val == String(manifest[i]).split("/").pop()){
					duplicate = true;
					break;
				}
			}
			if (!duplicate){
				//console.log("add to manifest: " + val);
				manifest.push(imagesDir + val);
			}
		}
	}
}

// preload images and sounds
function preloadAssets(){

	//add images to manifest
	$.each(data, recurse);

	// add audio to manifest
	var fileType = ".mp3";
	var obj = data.quiz.sounds;
	for (var key in obj) {
		var val = obj[key];
		//console.log(key + ": " + soundsDir + val + fileType);
		manifest.push({id: key, src: soundsDir + val + fileType});
	}
	
	//console.log("manifest(" + manifest.length + "): " + manifest.toString());
	createjs.Sound.registerPlugins([createjs.WebAudioPlugin, createjs.HTMLAudioPlugin]);//, createjs.FlashPlugin
	createjs.Sound.alternateExtensions = ["ogg"];//"mp3", "m4a", "wav"
	preload = new createjs.LoadQueue(true);//use false for local testing - true required for audio
	preload.installPlugin(createjs.Sound);
	preload.setMaxConnections(5);
	preload.on("fileload", handleFileLoad);
	preload.on("progress", handleOverallProgress);
	//preload.on("fileprogress", handleFileProgress);
	preload.on("error", handleFileError);
	loadAll();
}
	
function loadAll() {
	while (manifest.length > 0) {
		loadAnother();
	}
}

function loadAnother() {
	var item = manifest.shift();
	console.log(manifest.length, 'loading', item);
	preload.loadFile(item);
}

function handleFileLoad(event) {
	var img = event.result;
	console.log('loaded ' + event.item.id);
}

//function handleFileProgress(event) {

//}

function handleOverallProgress(event) {
	var percent = preload.progress * 100;
	//console.log('percent loaded: ' + percent);
	$('#quiz_preloader_bar').css('width', percent + '%');
	if (preload.progress == 1){
		//console.log("loading complete");
		$('#quiz_preloader').remove();
		if (preload != null) {
			preload.close();
		}
		init();
	}
}

function handleFileError(event) {
 	console.log('error loading ' + event.item.id + ': ' + event.target.error);
}

function init(){
	// shortcut
	questions = data.quiz.questions;

	// set images
	$('#quiz').css('background-image', 'url(' + imagesDir + data.quiz.backgroundImage + ')');
	$('#quiz_title').html("<img src='" + imagesDir + data.quiz.title.image + "' alt='" + data.quiz.title.alt + "' />");
	
	// show stuff
	$('#quiz_q_and_a').show();
	$('#quiz_image').show();
	$('#quiz_tracker').show();
	$('#quiz_question_number').show();
	$('#quiz_audio_toggle').show();
	
	// set up buttons
	$('#quiz_audio_toggle').mousedown(function(){
		toggleAudio();
	});
	$('#quiz_next').click(function(){
		onNext();
	});
	$('#quiz_try_again').click(function(){
		restart();
	});
	
	// generate tracker
	for (var i = 0; i < questions.length; i++){
		$('#quiz_tracker').append('<div><img /></div>');
	}
	$('#quiz_tracker img').attr('src', imagesDir + data.quiz.trackerImages.empty);
	$('#quiz_tracker div').css('width', 100 / questions.length + '%');
	$('#quiz_tracker').css('max-width', questions.length * 52);
	//$('#quiz_tracker').css({'left': '50%', 'margin-left': -$('#quiz_tracker').width()/2});
	
	// add answer numbers to answer button divs
	$('.quiz_answer').each(function(index, element) {
		$(element).data('quiz_answer_num', index + 1);
	});
	
	startQuiz();	
}

function startQuiz(){
	questionNum = 1;
	correctAnswers = 0;
	$('#quiz_tracker').show();
	populateQuiz();
	enableAnswers();
}

function populateQuiz(){
	$('.quiz_container_inner').css({'height': 'auto'});
	
	$('.quiz_question_number').text(questionNum);
	$('#quiz_question').html(questions[questionNum-1]["question"]);
	
	$('.quiz_answer').hide();
	$('.quiz_answer').each(function(index, element) {
		if (questions[questionNum-1]["answers"][index]){
			$(element).show().html("<strong>" + String.fromCharCode(97+index) + ".</strong> " + questions[questionNum-1]["answers"][index]);
		}
	});
	
	correctAnswer = questions[questionNum-1]["correctAnswer"];
	$("#quiz_image img").attr("src", imagesDir + questions[questionNum-1]["image"]);
	
	$('#quiz_feedback').hide();
}

function disableAnswers(){
	$('.quiz_answer').off();
	$('.quiz_answer').toggleClass('quiz_answer_enabled answerDisabled');
}

function enableAnswers(){
	$('.quiz_answer').removeClass('quiz_answer_disabled answer_selected');
	$('.quiz_answer').addClass('quiz_answer_enabled');
	$('.quiz_answer').on('click', function(e) {
		$(this).toggleClass('quiz_answer_disabled answer_selected');
		onAnswer($(this).data('quiz_answer_num'));
	});
}

function onAnswer(answer_num){
	//console.log(answer_num + ' vs ' + correctAnswer);
	
	disableAnswers();
	$('.quiz_answer').hide();
	
	//$('.quiz_container_inner').css({'height': $('.quiz_container_inner').height()});
	
	var info = questions[questionNum-1]["info"];	
	
	// if correct
	if (answer_num == correctAnswer){
		correctAnswers++;
		$('#quiz_question').text(questions[questionNum-1].correctResponse);
		$('#quiz_feedback').text(info);
		$('#quiz_tracker div:nth-child(' + questionNum + ') img').attr('src', imagesDir + data.quiz.trackerImages.right);
		playSound("correct");
	// if incorrect	
	} else {
		$('#quiz_question').text(questions[questionNum-1].incorrectResponse);
		//$('#quiz_feedback').html('The correct answer is ' + String.fromCharCode(96 + Number(correctAnswer)) + '.<br>' + info);
		$('#quiz_feedback').text(info);
		$('#quiz_tracker div:nth-child(' + questionNum + ') img').attr('src', imagesDir + data.quiz.trackerImages.wrong);
		playSound("incorrect");
	}
	
	$('#quiz_feedback').show();	
	$('#quiz_next').show();//.css({'top': $('#quiz_q_and_a').position().top + $('#quiz_q_and_a').height()});
	//.animate({height: "hide", width: "hide"}, 1500, "easeOutBack")
}

function onNext(){
	$('#quiz_next').hide();
	questionNum++;
	if (questionNum <= questions.length){
		populateQuiz();
		enableAnswers();
	} else {
		gameOver();
	}
}

function gameOver(){
	// add text to game ove popup
	var percentCorrect = Math.round(correctAnswers / questions.length * 100);
	var endHeader;
	if (percentCorrect <= 33){
		endHeader = data.quiz.gameOver.endHeader[0];
	} else if (percentCorrect <= 66){
		endHeader = data.quiz.gameOver.endHeader[1];
	} else {
		endHeader = data.quiz.gameOver.endHeader[2];
	}
	var endText = data.quiz.gameOver.endText.split('[correctCount]').join(correctAnswers).split('[questionCount]').join(questions.length).split('[percentCorrect]').join(percentCorrect);
	$('#quiz_popup_container p ').html(endHeader + '<br />' + endText);
	// fade and move stuff off the screen
	$('#quiz_tracker').fadeTo(250, 0, "easeInSine");
	$('#quiz_image').animate({left: -500}, 250, "easeInSine");
	$('#quiz_q_and_a').animate({top: '+=500'}, 250, "easeInSine", function(){
		$('#quiz_tracker').css('opacity', 0);
		$('#quiz_main').hide();
		// then fade in the final popup
		$('#quiz_popup').fadeIn();
	});
	playSound("gameOver");
	// Google analytics
	//trackEvent(category, action, opt_label, opt_value, opt_noninteraction)
	//_trackEvent("Kids", "Quizwhiz_complete", data.quiz.name);
	_gaq.push(['_trackEvent', 'Kids', 'Quizwhiz_complete', data.quiz.name]);
}

function restart(){
	// hide stuff
	$('#quiz_popup').hide();
	$('#endTitle').hide();
	// reset stuff
	$('#quiz_tracker').removeAttr('style');
	$('#quiz_tracker').css('max-width', questions.length * 52);
	$('#quiz_main').removeAttr('style');
	$('#quiz_image').removeAttr('style');
	$('#quiz_question_number').removeAttr('style');
	$('#quiz_q_and_a').removeAttr('style');
	$('#quiz_tracker img').attr('src', imagesDir + data.quiz.trackerImages.empty);
	// start over
	startQuiz();
}

function playSound(name, loop) {
	if (!muteSounds){
		console.log("playSound " + name);
		var instance = createjs.Sound.play(name, createjs.Sound.INTERRUPT_ANY);
		instance.addEventListener("failed", function(event){console.log("failed", event.target);});
		instance.addEventListener("complete", function(event){console.log("complete", event.target);});
	}
}

function toggleAudio(){
	if (muteSounds){
		muteSounds = false;
		createjs.Sound.setMute(false);
		$('#quiz_audio_toggle div').removeClass('fa-volume-off').addClass('fa-volume-up');
	} else {
		muteSounds = true;
		createjs.Sound.setMute(true);
		$('#quiz_audio_toggle div').removeClass('fa-volume-up').addClass('fa-volume-off');
	}
	console.log("muteSounds: " + muteSounds);
}

//}());